import 'package:flutter/material.dart';

const Duration _dialogSizeAnimationDuration = Duration(milliseconds: 200);
const double kPickerHeaderPortraitHeight = 80.0;
const double kPickerHeaderLandscapeWidth = 168.0;
const double kDialogActionBarHeight = 60.0;

/// This is a support widget that returns an Dialog with checkboxes as a Widget.
/// It is designed to be used in the showDialog method of other fields.
class ResponsiveDialog extends StatefulWidget {
  ResponsiveDialog({
    required this.context,
    String? title,
    Widget? child,
    this.headerColor,
    this.backgroundColor,
    this.forcePortrait = false,
    double? maxLongSide,
    double? maxShortSide,
    this.hideButtons = false,
    this.okPressed,
    this.cancelPressed,
    this.confirmText,
    this.cancelText,
    this.headerTextStyle,
    this.confirmTextStyle,
    this.cancelTextStyle,
    this.bottomBorderColor,
  })  : title = title ?? "Title Here",
        child = child ?? Text("Content Here"),
        maxLongSide = maxLongSide ?? 600,
        maxShortSide = maxShortSide ?? 400;

  final BuildContext context;
  final String? title;
  final Widget child;
  final bool forcePortrait;
  final Color? headerColor;
  final TextStyle? headerTextStyle;
  final TextStyle? confirmTextStyle;
  final TextStyle? cancelTextStyle;
  final Color? backgroundColor;
  final Color? bottomBorderColor;
  final double? maxLongSide;
  final double? maxShortSide;
  final String? confirmText;
  final String? cancelText;
  final bool? hideButtons;

  // Events
  final VoidCallback? cancelPressed;
  final VoidCallback? okPressed;

  @override
  _ResponsiveDialogState createState() => _ResponsiveDialogState();
}

class _ResponsiveDialogState extends State<ResponsiveDialog> {
  late Color _headerColor;
  late Color _backgroundColor;

  Widget header(BuildContext context, Orientation orientation) {
    return Container(
      color: _headerColor,
      height: (orientation == Orientation.portrait)
          ? kPickerHeaderPortraitHeight
          : null,
      width: (orientation == Orientation.landscape)
          ? kPickerHeaderLandscapeWidth
          : null,
      child: Align(
        alignment: Alignment.centerLeft,
        child: Text(
          widget.title!,
          style: widget.headerTextStyle,
        ),
      ),
      padding: EdgeInsets.all(20.0),
    );
  }

  Widget actionBar(BuildContext context) {
    var localizations = MaterialLocalizations.of(context);
    return Container(
      height: kDialogActionBarHeight,
      child: Container(
        decoration: BoxDecoration(
          border: Border(
            top: BorderSide(
                width: 0.2, color: widget.bottomBorderColor ?? Colors.grey),
          ),
        ),
        child: ButtonBar(
          children: <Widget>[
            TextButton(
              child: Text(
                widget.cancelText ?? localizations.cancelButtonLabel,
                style: widget.cancelTextStyle,
              ),
              onPressed: () => (widget.cancelPressed == null)
                  ? Navigator.of(context).pop()
                  : widget.cancelPressed!(),
            ),
            TextButton(
              child: Text(
                widget.confirmText ?? localizations.okButtonLabel,
                style: widget.confirmTextStyle,
              ),
              onPressed: () => (widget.okPressed == null)
                  ? Navigator.of(context).pop()
                  : widget.okPressed!(),
            ),
          ],
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);
    _headerColor = widget.headerColor ?? theme.primaryColor;
    _backgroundColor = widget.backgroundColor ?? theme.dialogBackgroundColor;

    final Orientation orientation = MediaQuery.of(context).orientation;

    // constrain the dialog from expanding to full screen
    final Size dialogSize = (orientation == Orientation.portrait)
        ? Size(widget.maxShortSide!, widget.maxLongSide!)
        : Size(widget.maxLongSide!, widget.maxShortSide!);

    return Dialog(
      backgroundColor: _backgroundColor,
      child: AnimatedContainer(
        width: dialogSize.width,
        height: dialogSize.height,
        duration: _dialogSizeAnimationDuration,
        child: OrientationBuilder(
          builder: (BuildContext context, Orientation orientation) {
            if (widget.forcePortrait) orientation = Orientation.portrait;

            switch (orientation) {
              case Orientation.portrait:
                return Column(
                  children: <Widget>[
                    header(context, orientation),
                    Expanded(
                      child: Container(
                        child: widget.child,
                      ),
                    ),
                    actionBar(context),
                  ],
                );
              case Orientation.landscape:
                return Row(
                  children: <Widget>[
                    header(context, orientation),
                    Expanded(
                      child: Column(
                        children: <Widget>[
                          Expanded(
                            child: widget.child,
                          ),
                          actionBar(context),
                        ],
                      ),
                    ),
                  ],
                );
            }
          },
        ),
      ),
    );
  }
}
