import 'package:flutter/material.dart';
import '../scroll_picker.dart';
import 'responsive_dialog.dart';

class ScrollPickerDialog<T> extends StatefulWidget {
  ScrollPickerDialog({
    this.title,
    required this.items,
    required this.selectedItem,
    this.transformer,
    this.headerColor,
    this.backgroundColor,
    this.maxLongSide,
    this.maxShortSide,
    this.confirmText,
    this.cancelText,
    this.headerTextStyle,
    this.confirmTextStyle,
    this.cancelTextStyle,
    this.bottomBorderColor,
    this.hideButtons,
    this.scrollItemTextStyle,
    this.selectedItemTextStyle,
  });

  final List<T> items;
  final T? selectedItem;
  final String? Function(T item)? transformer;
  final String? title;

  final Color? headerColor;
  final TextStyle? headerTextStyle;
  final TextStyle? confirmTextStyle;
  final TextStyle? cancelTextStyle;
  final TextStyle? scrollItemTextStyle;
  final TextStyle? selectedItemTextStyle;
  final Color? backgroundColor;
  final Color? bottomBorderColor;
  final double? maxLongSide;
  final double? maxShortSide;
  final String? confirmText;
  final String? cancelText;
  final bool? hideButtons;

  @override
  State<ScrollPickerDialog> createState() =>
      _ScrollPickerDialogState<T>(selectedItem ?? items[0]);
}

class _ScrollPickerDialogState<T> extends State<ScrollPickerDialog<T>> {
  _ScrollPickerDialogState(this.selectedItem);

  T selectedItem;

  @override
  Widget build(BuildContext context) {
    return ResponsiveDialog(
      context: context,
      title: widget.title,
      headerColor: widget.headerColor,
      backgroundColor: widget.backgroundColor,
      headerTextStyle: widget.headerTextStyle,
      confirmTextStyle: widget.confirmTextStyle,
      cancelTextStyle: widget.cancelTextStyle,
      maxLongSide: widget.maxLongSide,
      maxShortSide: widget.maxLongSide,
      confirmText: widget.confirmText,
      cancelText: widget.cancelText,
      child: ScrollPicker<T>(
        items: widget.items,
        selectedItem: selectedItem,
        onChanged: (value) => setState(() => selectedItem = value),
        transformer: widget.transformer,
        scrollItemTextStyle: widget.scrollItemTextStyle,
        selectedItemTextStyle: widget.selectedItemTextStyle,
        bottomBorderColor: widget.bottomBorderColor,
      ),
      okPressed: () => Navigator.of(context).pop(selectedItem),
    );
  }
}
