import 'package:flutter/material.dart';

class ScrollPicker<T> extends StatefulWidget {
  ScrollPicker({
    Key? key,
    required this.items,
    required this.selectedItem,
    required this.onChanged,
    this.transformer,
    this.bottomBorderColor,
    this.scrollItemTextStyle,
    this.selectedItemTextStyle,
  }) : super(key: key);

  final ValueChanged<T> onChanged;

  final List<T> items;
  final T selectedItem;

  final Color? bottomBorderColor;
  final TextStyle? scrollItemTextStyle;
  final TextStyle? selectedItemTextStyle;

  final String? Function(T item)? transformer;

  @override
  _ScrollPickerState createState() => _ScrollPickerState<T>(selectedItem);
}

class _ScrollPickerState<T> extends State<ScrollPicker<T>> {
  _ScrollPickerState(this.selectedValue);

  // Constants
  static const double itemHeight = 50.0;

  // Variables
  late double widgetHeight;
  late int numberOfVisibleItems;
  late int numberOfPaddingRows;
  late double visibleItemsHeight;
  late double offset;

  T selectedValue;

  late ScrollController scrollController;

  @override
  void initState() {
    super.initState();

    int initialItem = widget.items.indexOf(selectedValue);
    scrollController = FixedExtentScrollController(initialItem: initialItem);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        widgetHeight = constraints.maxHeight;

        return Stack(
          children: <Widget>[
            GestureDetector(
              onTapUp: _itemTapped,
              child: ListWheelScrollView.useDelegate(
                childDelegate: ListWheelChildBuilderDelegate(
                    builder: (BuildContext context, int index) {
                  if (index < 0 || index > widget.items.length - 1) {
                    return null;
                  }

                  final value = widget.items[index];

                  return Center(
                    child: Text(
                      widget.transformer?.call(value) ?? '$value',
                      style: widget.selectedItem == value
                          ? widget.selectedItemTextStyle
                          : widget.scrollItemTextStyle?.copyWith(),
                    ),
                  );
                }),
                controller: scrollController,
                itemExtent: itemHeight,
                onSelectedItemChanged: _onSelectedItemChanged,
                physics: FixedExtentScrollPhysics(),
              ),
            ),
            Center(
              child: Container(
                height: itemHeight,
                decoration: BoxDecoration(
                  border: Border(
                    top: BorderSide(
                        color: widget.bottomBorderColor ?? Colors.grey,
                        width: 1.0),
                    bottom: BorderSide(
                        color: widget.bottomBorderColor ?? Colors.grey,
                        width: 1.0),
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }

  void _itemTapped(TapUpDetails details) {
    Offset position = details.localPosition;
    double center = widgetHeight / 2;
    double changeBy = position.dy - center;
    double newPosition = scrollController.offset + changeBy;

    // animate to and center on the selected item
    scrollController.animateTo(newPosition,
        duration: Duration(milliseconds: 500), curve: Curves.easeInOut);
  }

  void _onSelectedItemChanged(int index) {
    T newValue = widget.items[index];
    if (newValue != selectedValue) {
      selectedValue = newValue;
      widget.onChanged(newValue);
    }
  }
}
